<?php

namespace Drupal\views_extras\Plugin\views\argument_default;

use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Utility\Token;
use Drupal\views\Plugin\views\argument_default\ArgumentDefaultPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Default argument plugin to use the raw value from the URL.
 *
 * @ingroup views_argument_default_plugins
 *
 * @ViewsArgumentDefault(
 *   id = "tempstore",
 *   title = @Translation("TempStore variable")
 * )
 */
class TempStore extends ArgumentDefaultPluginBase implements CacheableDependencyInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected AccountProxyInterface $currentUser;

  /**
   * Module Handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * Token Handler.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected Token $token;

  /**
   * Token Handler.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected PrivateTempStoreFactory $tempStoreFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->currentUser = $container->get('current_user');
    $instance->moduleHandler = $container->get('module_handler');
    $instance->token = $container->get('token');
    $instance->tempStoreFactory = $container->get('tempstore.private');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['tempStore_unique_name'] = ['default' => ''];
    $options['tempStore_key'] = ['default' => ''];
    $options['fallback_value'] = ['default' => FALSE];
    $options['cache_time'] = ['default' => -1];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $form['tempStore_unique_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Unique Name'),
      '#default_value' => $this->options['tempStore_unique_name'],
      '#description' => $this->t('Unique namespace, used while setting the tempStore'),
    ];
    $form['tempStore_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('TempStore key'),
      '#default_value' => $this->options['tempStore_key'],
      '#description' => $this->t('Keys of tempStore variable'),
    ];
    $form['fallback_value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('If tempStore is not set, what should be the fallback value.'),
      '#default_value' => $this->options['fallback_value'],
      '#description' => $this->t('You may use user tokens.'),
    ];
    if ($this->moduleHandler->moduleExists("token")) {
      $form['token_help'] = [
        '#type' => 'markup',
        '#token_types' => ['user'],
        '#theme' => 'token_tree_link',
      ];
    }
    $form['cache_time'] = [
      '#type' => 'number',
      '#title' => $this->t('Cache Maximum Age.'),
      '#default_value' => $this->options['cache_time'],
      '#description' => $this->t('If tempStore value changes set it to 0.'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getArgument() {
    if (!empty($this->options['tempStore_key'])) {
      if ($value = $this->tempStoreFactory->get($this->options['tempStore_unique_name'])->get($this->options['tempStore_key'])) {
        return $value;
      }
    }
    if (!empty($value = $this->options['fallback_value'])) {
      return $this->token->replace($value, ['user' => $this->currentUser]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return (int) $this->options['cache_time'];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return ['session'];
  }

}
